.file-upload-field-container {
    margin: 10px 0;
    position: relative;
    border-bottom: 1px solid #e4e4e4;

    ul {
        padding-bottom: 10px;
        > li {
            width: 300px;
            padding: 2px 5px;
            margin: 0;

            label {
                width: 125px;
            }
            input, select {
                width: 150px;
            }
            select {
                display: inline-block;
            }
        }
    }
    .remove-btn{
        float: right;
        line-height: 14px;
        font-size: 80%;
        position: absolute;
        top: 5px;
        right: 5px;
    }
}

.file-upload-fields-container {
    margin: 0 0 0 35px;

    .add-btn{
        line-height: 14px;
        font-size: 80%;
    }
}