.survey-invite {
  text-align: center;
  font-family:"Arial Black",Gadget,sans-serif;
  padding: 10px;

  img {
    max-width:350px;
    margin: 0 auto;
  }

  .survey-api-title {
    font-weight: 900;
    padding: 10px;
  }

  .survey-api-desc {
    color: #2F5790;
  }
}

.survey-disclaimer {
  text-align: center;
  font-size: 0.8em;
}

.survey-no-api {
  font-weight: 700;
  color: #C13333;
}
